

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

The #PangoAttrString structure is used to represent attributes with
a string value.
-}

module GI.Pango.Structs.AttrString
    ( 

-- * Exported types
    AttrString(..)                          ,
    newZeroAttrString                       ,
    noAttrString                            ,


 -- * Properties
-- ** Attr
    attrStringReadAttr                      ,


-- ** Value
    attrStringReadValue                     ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Pango.Types
import GI.Pango.Callbacks

newtype AttrString = AttrString (ForeignPtr AttrString)
-- | Construct a `AttrString` struct initialized to zero.
newZeroAttrString :: MonadIO m => m AttrString
newZeroAttrString = liftIO $ callocBytes 24 >>= wrapPtr AttrString

noAttrString :: Maybe AttrString
noAttrString = Nothing

attrStringReadAttr :: AttrString -> IO Attribute
attrStringReadAttr s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO (Ptr Attribute)
    val' <- (newPtr 16 Attribute) val
    return val'

attrStringReadValue :: AttrString -> IO T.Text
attrStringReadValue s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO CString
    val' <- cstringToText val
    return val'

type family ResolveAttrStringMethod (t :: Symbol) (o :: *) :: * where
    ResolveAttrStringMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveAttrStringMethod t AttrString, MethodInfo info AttrString p) => IsLabelProxy t (AttrString -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveAttrStringMethod t AttrString, MethodInfo info AttrString p) => IsLabel t (AttrString -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


